@@ -3,6 +3,8 @@ from typing import Callable
 from logic_bank.exec_row_logic.logic_row import LogicRow
 from logic_bank.rule_type.row_event import EarlyRowEvent
 
+# see logicbank - examples/payment_allocation
+# not currently tested in ApiLogicServer...
 
 class Allocate(EarlyRowEvent):
     """
@@ -60,7 +62,7 @@ class Allocate(EarlyRowEvent):
                                                                  provider)
             if not allocator:
                 break
-        provider.log(f'END {str(self)}')
+        provider.log(f'END {str(self)}')  # last log for payment_allo example
         return self
 
     def while_calling_allocator_default(self, allocation_logic_row, provider_logic_row) -> bool:
